package com.cusc.nirvana.user.rnr.customer.controller;


import com.cusc.nirvana.common.result.Response;

import com.cusc.nirvana.user.rnr.customer.dto.FileDownloadDTO;
import com.cusc.nirvana.user.rnr.customer.service.IFileService;

import com.cusc.nirvana.user.rnr.fp.client.FileSystemClient;
import com.cusc.nirvana.user.rnr.fp.dto.FileRecordDTO;
import com.cusc.nirvana.user.rnr.fp.dto.FileUploadDTO;
import com.cusc.nirvana.user.rnr.fp.dto.ImageBase64DTO;
import com.cusc.nirvana.web.exception.AppGlobalException;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.URLEncoder;
import java.util.Base64;

/**
 * @author stayAnd
 * @date 2022/4/12
 */
@Slf4j
@RestController
@RequestMapping("/file")
public class FileController {


	@Autowired
	private IFileService fileService;

	@Resource
	private FileSystemClient fileSystemClient;

	@PostMapping("/upload")
	public Response<FileRecordDTO> upload(FileUploadDTO fileUploadDTO){
		//直接调用文件系统，将公有改为私有
//		fileUploadDTO.setType("private");
//		if (StringUtils.isBlank(fileUploadDTO.getUserId())) {
//			fileUploadDTO.setUserId(UserSubjectUtil.getUserId());
//		}
//		return fileService.upload(fileUploadDTO);

		//文件大小必须大于0
		if (fileUploadDTO.getFile().getSize() < 1) {
			return Response.createError("文件内容不能为空");
		}

		return fileService.uploadTwo(fileUploadDTO);
	}

    /**
     * 文件下载
     * @param fileDownloadDTO
     * @return
     */
//    @PostMapping("/download")
//    public Response download(HttpServletResponse response, @RequestBody FileDownloadDTO fileDownloadDTO) {
//        OutputStream outputStream = null;
//        InputStream inputStream = null;
//        File file = null;
//        try {
//            /*file = fileService.downloadToLocal(fileDownloadDTO);
//			response.getOutputStream().write();
//			if(file == null){
//                throw new Exception("文件不存在");
//            }
//            inputStream = new FileInputStream(file);
//            // 以文件形式下载
//            response.setHeader("Content-Disposition",
//                "attachment;filename=" + URLEncoder.encode(StringUtils.isNotEmpty(fileDownloadDTO.getFileName()) ? fileDownloadDTO.getFileName() : "文件.xlsx", "utf-8"));
//            response.setHeader("Access-Control-Expose-Headers","Content-Disposition");
//
//            BufferedInputStream in = new BufferedInputStream(inputStream);
//            outputStream = new BufferedOutputStream(response.getOutputStream());
//            byte[] car = new byte[1024];
//            int len;
//            while ((len = in.read(car)) != -1) {
//                outputStream.write(car, 0, len);
//            }
//            outputStream.flush();*/
//
//			response.setHeader("Content-Disposition",
//					"attachment;filename=" + URLEncoder.encode(StringUtils.isNotEmpty(fileDownloadDTO.getFileName()) ? fileDownloadDTO.getFileName() : "文件.xlsx", "utf-8"));
//			response.setHeader("Access-Control-Expose-Headers","Content-Disposition");
//            response.getOutputStream().write(fileService.downloadToByteArray(fileDownloadDTO));
//        } catch (Exception e) {
//            throw new AppGlobalException("下载失败", e);
//        } finally {
//            /*IOUtils.closeQuietly(inputStream);
//            IOUtils.closeQuietly(outputStream);
//            if (file != null && file.exists() && file.isFile()) {
//                file.delete();
//            }*/
//			try {
//				response.getOutputStream().close();
//			} catch (IOException e) {
//				e.printStackTrace();
//			}
//        }
//
//        return Response.createSuccess();
//    }


	/**
	 * 下载接口
	 * @param response
	 * @param fileDownloadDTO
	 * @return
	 */
	@PostMapping("/download")
	public Response download(HttpServletResponse response, @RequestBody FileDownloadDTO fileDownloadDTO) {
		try {
			response.setHeader("Content-Disposition",
					"attachment;filename=" + URLEncoder.encode(StringUtils.isNotEmpty(fileDownloadDTO.getFileName()) ? fileDownloadDTO.getFileName() : "文件.xlsx", "utf-8"));
			response.setHeader("Access-Control-Expose-Headers", "Content-Disposition");

			byte[] result = null;
			try {
				log.info("下载文件-开始, uuid : {}", fileDownloadDTO.getUuid());

				com.cusc.nirvana.user.rnr.fp.dto.FileDownloadDTO dto = new com.cusc.nirvana.user.rnr.fp.dto.FileDownloadDTO();
				dto.setUuid(fileDownloadDTO.getUuid());
				Response<ImageBase64DTO> base64Response = fileSystemClient.getBase64(dto);

				String base64 = base64Response.getData().getBase64();

				result = Base64.getDecoder().decode(base64);

				log.info("下载文件-结束, uuid : {}", fileDownloadDTO.getUuid());
			} catch (Exception e) {
				log.error("getImageBase64 error: {}", e.getMessage(), e);
				return Response.createError(e.getMessage());
			}
			//InputStream fileStream = MinIOUtils.getObject(minIOConfig.getBucketName(), fileDownloadDTO.getUuid());
			response.getOutputStream().write(result);
		} catch (Exception e) {
			throw new AppGlobalException("下载失败", e);
		} finally {
			try {
				if (response.getOutputStream() != null) {
					response.getOutputStream().close();
				}
			} catch (IOException e) {
				log.error("---response.getOutputStream().close()--exception--", e);
			}
		}
		return Response.createSuccess();
	}



}
