package com.cusc.nirvana.user.rnr.customer.controller;

import com.alibaba.fastjson.JSONObject;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.auth.client.CaptchaClient;
import com.cusc.nirvana.user.auth.client.CiamLoginClient;
import com.cusc.nirvana.user.auth.client.LoginClient;
import com.cusc.nirvana.user.auth.common.constants.CaptchaTypeEnum;
import com.cusc.nirvana.user.auth.common.constants.UserTypeEnum;
import com.cusc.nirvana.user.auth.identification.dto.CaptchaCreateReq;
import com.cusc.nirvana.user.auth.identification.dto.CaptchaCreateResp;
import com.cusc.nirvana.user.auth.identification.dto.MobileLoginReq;
import com.cusc.nirvana.user.auth.identification.dto.Oauth2Token;
import com.cusc.nirvana.user.auth.identification.dto.SmsSendConfig;
import com.cusc.nirvana.user.exception.CuscUserException;
import com.cusc.nirvana.user.rnr.customer.config.CustomerConfig;
import com.cusc.nirvana.user.rnr.customer.constants.CustomerConstant;
import com.cusc.nirvana.user.rnr.customer.constants.SmsConfigConstant;
import com.cusc.nirvana.user.rnr.customer.dto.RnrLoginReqDTO;
import com.cusc.nirvana.user.rnr.customer.util.ValidationUtil;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * Description: c端客户登录控制层
 * <br />
 * CreateDate 2022-04-24 17:30:21
 *
 * @author yuyi
 **/
@RestController
@RequestMapping("/login")
@ApiModel("B端用户登录相关(车企、经销商)")
@Slf4j
public class LoginController {

    @Autowired
    private LoginClient loginClient;

    @Autowired
    private CiamLoginClient ciamLoginClient;

    @Autowired
    private CaptchaClient captchaClient;
    @Autowired
    private CustomerConfig customerConfig;

    @GetMapping("/getGraphic")
    @ApiOperation("获取图片验证码")
    public Response<CaptchaCreateResp> getGraphic() {
        CaptchaCreateReq req = new CaptchaCreateReq();
        req.setApplicationId(CustomerConstant.APP_ID);
        req.setCaptchaHeight(40);
        req.setCaptchaWidth(90);
        req.setCaptchaType(CaptchaTypeEnum.TYPE_DEFAULT);
        req.setCaptchaLength(4);
        Response<CaptchaCreateResp> response = captchaClient.simpleGraphic(req);
        if (null == response || !response.isSuccess() || null == response.getData()) {
            return Response.createError(response.getMsg(), response.getCode());
        }
        return Response.createSuccess(response.getData());
    }


    @PostMapping("/getSms")
    @ApiOperation("获取短信验证码")
    public Response getSms(@RequestBody MobileLoginReq bean) {
		//检查手机
		Response checkPhone = ValidationUtil.checkPhone(bean.getPhone());
		if (!checkPhone.isSuccess()) {
			throw new CuscUserException(checkPhone.getCode(),checkPhone.getMsg());
		}

        bean.setApplicationId(CustomerConstant.APP_ID);
        bean.setCheckCaptchaImg(true);
        bean.setCaptchaExpire(SmsConfigConstant.SMS_CAPTCHA_EXPIRE_TIME);

        SmsSendConfig smsConfig = new SmsSendConfig();
        smsConfig.setStrategyCode(SmsConfigConstant.SMS_STRATEGY_CODE);
        smsConfig.setSmsTemplateCode(SmsConfigConstant.SMS_TEMPLATE_CODE);
        bean.setSmsSendConfig(smsConfig);
        bean.setUserType(UserTypeEnum.CIAM.getCode());
        log.info("getSms request = {}", JSONObject.toJSONString(bean));
        bean.setTenantNo(customerConfig.getTenantNo());
        return loginClient.sendSmsCaptcha(bean);
    }

    @PostMapping("/getSmsH5")
    @ApiOperation("获取短信验证码")
    public Response getSmsH5(@RequestBody MobileLoginReq bean) {
		//检查手机
		Response checkPhone = ValidationUtil.checkPhone(bean.getPhone());
		if (!checkPhone.isSuccess()) {
			throw new CuscUserException(checkPhone.getCode(),checkPhone.getMsg());
		}

        bean.setApplicationId(CustomerConstant.APP_ID);
        bean.setCheckCaptchaImg(true);
        bean.setCaptchaExpire(SmsConfigConstant.SMS_CAPTCHA_EXPIRE_TIME);

        SmsSendConfig smsConfig = new SmsSendConfig();
        smsConfig.setStrategyCode(SmsConfigConstant.SMS_STRATEGY_CODE);
        smsConfig.setSmsTemplateCode(SmsConfigConstant.SMS_TEMPLATE_CODE);
        bean.setSmsSendConfig(smsConfig);
        bean.setUserType(UserTypeEnum.CIAM.getCode());
        //允许没注册校验
        bean.setLoginType("allow");
        log.info("getSms request = {}", JSONObject.toJSONString(bean));
        bean.setTenantNo(customerConfig.getTenantNo());
        return loginClient.sendSmsCaptchaNew(bean);
    }

    @PostMapping("/ciam")
    @ApiOperation("登录")
    public Response<Oauth2Token> login(@RequestBody RnrLoginReqDTO bean) {
		//检查手机
		Response checkPhone = ValidationUtil.checkPhone(bean.getPhone());
		if (!checkPhone.isSuccess()) {
			throw new CuscUserException(checkPhone.getCode(),checkPhone.getMsg());
		}
        MobileLoginReq mobileLogin = new MobileLoginReq();
        mobileLogin.setApplicationId(CustomerConstant.APP_ID);
        mobileLogin.setPhone(bean.getPhone());
        mobileLogin.setCaptcha(bean.getSms());
        mobileLogin.setTenantNo(customerConfig.getTenantNo());
        mobileLogin.setUserType(UserTypeEnum.CIAM.getCode());
        return ciamLoginClient.mobileLogin(mobileLogin);
    }

    /**
     * 小鹏汽车登录接口
     * @param bean
     * @return
     */
    @PostMapping("/ciamXP")
    @ApiOperation("登录")
    public Response<Oauth2Token> loginXP(@RequestBody RnrLoginReqDTO bean) {
        log.info("ciamXP reqStr:"+JSONObject.toJSONString(bean));
        MobileLoginReq mobileLogin = new MobileLoginReq();
        mobileLogin.setApplicationId(CustomerConstant.APP_ID);
        mobileLogin.setPhone(bean.getPhone());
        mobileLogin.setTenantNo(customerConfig.getTenantNo());
        mobileLogin.setUserType(UserTypeEnum.CIAM.getCode());
        log.info("mobileLoginXP reqStr:"+JSONObject.toJSONString(mobileLogin));
        return ciamLoginClient.mobileLoginXP(mobileLogin);
    }

    @PostMapping("/userLoginAdd")
    @ApiOperation("注册登录")
    public Response<Oauth2Token> userLoginAdd(@RequestBody RnrLoginReqDTO bean) {
		//检查手机
		Response checkPhone = ValidationUtil.checkPhone(bean.getPhone());
		if (!checkPhone.isSuccess()) {
			throw new CuscUserException(checkPhone.getCode(),checkPhone.getMsg());
		}

        MobileLoginReq mobileLogin = new MobileLoginReq();
        mobileLogin.setApplicationId(CustomerConstant.APP_ID);
        mobileLogin.setPhone(bean.getPhone());
        mobileLogin.setCaptcha(bean.getSms());
//        mobileLogin.setTenantNo(bean.getTenantNo());
        mobileLogin.setTenantNo(customerConfig.getTenantNo());
        mobileLogin.setUserType(UserTypeEnum.CIAM.getCode());
        return ciamLoginClient.userLoginAdd(mobileLogin);
    }
}
