package com.cusc.nirvana.user.rnr.customer.controller;

import com.cusc.nirvana.common.result.PageResult;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.auth.authentication.plug.user.UserSubjectUtil;
import com.cusc.nirvana.user.auth.client.CaptchaClient;
import com.cusc.nirvana.user.auth.identification.dto.CaptchaVerificationReq;
import com.cusc.nirvana.user.rnr.customer.constants.CustomerConstant;
import com.cusc.nirvana.user.rnr.customer.constants.ResponseCode;
import com.cusc.nirvana.user.rnr.customer.converter.RnrRecordConverter;
import com.cusc.nirvana.user.rnr.customer.dto.RnrInfoReqDTO;
import com.cusc.nirvana.user.rnr.customer.dto.RnrInfoRespDTO;
import com.cusc.nirvana.user.rnr.customer.dto.RnrRecordPageReqDTO;
import com.cusc.nirvana.user.rnr.customer.dto.RnrRecordRespDTO;
import com.cusc.nirvana.user.rnr.customer.dto.RnrSmsReqDTO;
import com.cusc.nirvana.user.rnr.customer.dto.RnrUnboundReqDTO;
import com.cusc.nirvana.user.rnr.customer.service.ISmsService;
import com.cusc.nirvana.user.rnr.fp.dto.SmsRequestDTO;
import com.cusc.nirvana.user.rnr.mg.client.MgRnrCardInfoClient;
import com.cusc.nirvana.user.rnr.mg.client.MgRnrInfoClient;
import com.cusc.nirvana.user.rnr.mg.client.MgRnrUnboundClient;
import com.cusc.nirvana.user.rnr.mg.constants.IsBatchEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrBizzTypeEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrOrderAuditTypeEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrOrderSourceEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrOrderStatusEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrOrderType;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrCardInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.RnrOrderDTO;
import com.cusc.nirvana.user.rnr.mg.dto.VehicleUnbindDTO;
import com.cusc.nirvana.user.util.CuscStringUtils;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Description: C端用户实名记录
 * <br />
 * CreateDate 2022-04-24 17:34:24
 *
 * @author yuyi
 **/
@Slf4j
@RestController
@RequestMapping("/rnr")
public class RnrRecordController {

    @Autowired
    private MgRnrCardInfoClient rnrCardInfoClient;

    @Autowired
    private MgRnrInfoClient rnrInfoClient;

    @Autowired
    private MgRnrUnboundClient unboundClient;

    @Autowired
    private ISmsService smsService;

    @Autowired
    private CaptchaClient captchaClient;

    @PostMapping("/record")
    @ApiOperation(value = "实名认证记录", notes = "实名认证记录")
    public Response<PageResult<RnrRecordRespDTO>> record(@RequestBody RnrRecordPageReqDTO bean) {
        //通过用户id分页查询
        MgRnrCardInfoDTO card = new MgRnrCardInfoDTO();
        card.setCurrPage(bean.getCurrPage());
        card.setPageSize(bean.getPageSize());
        card.setUserId(UserSubjectUtil.getUserId());
        card.setTenantNo(UserSubjectUtil.getTenantNo());
        card.setRnrBizzType(RnrBizzTypeEnum.Bind.getCode());
        Response<PageResult<MgRnrCardInfoDTO>> cardResp = rnrCardInfoClient.queryPageByUserId(card);
        if (cardResp != null && cardResp.isSuccess()) {
            PageResult<MgRnrCardInfoDTO> cardPage = cardResp.getData();
            PageResult<RnrRecordRespDTO> ret;
            if (!CollectionUtils.isEmpty(cardPage.getList())) {
                List<RnrRecordRespDTO> dataList =
                        RnrRecordConverter.INSTANCE.cardListToRnrRecordList(cardPage.getList());
                ret = new PageResult(dataList, cardPage.getTotalCount(), cardPage.getPageSize(),
                        cardPage.getCurrPage());
            } else {
                ret = new PageResult(new ArrayList(), cardPage.getTotalCount(), cardPage.getPageSize(),
                        cardPage.getCurrPage());
            }
            return Response.createSuccess(ret);
        }
        return Response.createSuccess(new PageResult<>(bean.getCurrPage(), bean.getPageSize()));
    }

    @PostMapping("/rnrInfo")
    @ApiOperation(value = "实名记录查询", notes = "实名记录查询")
    public Response<RnrInfoRespDTO> rnrInfo(@RequestBody RnrInfoReqDTO bean) {
        if (CuscStringUtils.isEmpty(bean.getRnrId())) {
            return Response.createError(ResponseCode.PARAMETER_NOT_NULL.getMsg(),
                    ResponseCode.PARAMETER_NOT_NULL.getCode());
        }
        MgRnrInfoDTO info = new MgRnrInfoDTO();
        info.setUuid(bean.getRnrId());
        info.setTenantNo(UserSubjectUtil.getTenantNo());
        Response<MgRnrInfoDTO> infoResp = rnrInfoClient.getByUuid(info);
        RnrInfoRespDTO ret = null;
        if (infoResp != null && infoResp.isSuccess()) {
            ret = RnrRecordConverter.INSTANCE.rnrInfoToViewInfo(infoResp.getData());
        }
        return Response.createSuccess(ret);
    }

    @PostMapping("/unboundSms")
    @ApiOperation(value = "实名解绑短信", notes = "实名解绑短信")
    public Response unboundSms(@RequestBody RnrSmsReqDTO bean) {
        if (CuscStringUtils.isEmpty(bean.getRnrId())) {
            return Response.createError(ResponseCode.PARAMETER_NOT_NULL.getMsg(),
                    ResponseCode.PARAMETER_NOT_NULL.getCode());
        }
        //验证图形验证码
        CaptchaVerificationReq captchaVerificationReq = new CaptchaVerificationReq();
        captchaVerificationReq.setApplicationId(CustomerConstant.APP_ID);
        captchaVerificationReq.setCaptchaValue(bean.getImgCaptcha());
        captchaVerificationReq.setRequestId(bean.getRequestId());
        captchaVerificationReq.setTenantNo(UserSubjectUtil.getTenantNo());
        Response<Boolean> captchaResp = captchaClient.verificationCaptcha(captchaVerificationReq);
        if (captchaResp == null || !captchaResp.isSuccess() || !captchaResp.getData()) {
            return Response.createError(ResponseCode.IMG_CAPTCHA_INVALID.getMsg(),
                    ResponseCode.IMG_CAPTCHA_INVALID.getCode());
        }
        MgRnrInfoDTO info = new MgRnrInfoDTO();
        info.setUuid(bean.getRnrId());
        info.setTenantNo(UserSubjectUtil.getTenantNo());
        Response<MgRnrInfoDTO> infoResp = rnrInfoClient.getByUuid(info);
        if (infoResp != null && infoResp.isSuccess() && CuscStringUtils.isNotEmpty(infoResp.getData().getPhone())) {
            SmsRequestDTO smsReq = new SmsRequestDTO();
            smsReq.setBizType("rnr");
            smsReq.setPhone(infoResp.getData().getPhone());
            smsService.sendSmsRnr(smsReq);
            return Response.createSuccess();
        }
        return Response.createError(ResponseCode.RNR_PHONE_NOT_FOUND.getMsg(),
                ResponseCode.RNR_PHONE_NOT_FOUND.getCode());
    }

    @PostMapping("/unbound")
    @ApiOperation(value = "实名解绑", notes = "实名解绑")
    public Response unbound(@RequestBody RnrUnboundReqDTO bean) {
        if (CuscStringUtils.isEmpty(bean.getRnrId())
                || CuscStringUtils.isEmpty(bean.getIccid())) {
            return Response.createError(ResponseCode.PARAMETER_NOT_NULL.getMsg(),
                    ResponseCode.PARAMETER_NOT_NULL.getCode());
        }
        //查询手机号
        MgRnrInfoDTO info = new MgRnrInfoDTO();
        info.setUuid(bean.getRnrId());
        info.setTenantNo(UserSubjectUtil.getTenantNo());
        Response<MgRnrInfoDTO> infoResp = rnrInfoClient.getByUuid(info);
        if (infoResp == null || !infoResp.isSuccess() || infoResp.getData() == null || CuscStringUtils.isEmpty(
                infoResp.getData().getPhone())) {
            return Response.createError(ResponseCode.RNR_INFO_INVALID.getMsg(),
                    ResponseCode.RNR_INFO_INVALID.getCode());
        }
        info = infoResp.getData();
//        //检查短信验证码是否正确
//        SmsRequestDTO sms = new SmsRequestDTO();
//        sms.setBizType("rnr");
//        sms.setCaptcha(bean.getSmsCaptcha());
//        sms.setPhone(info.getPhone());
//        Response smsResp = smsService.checkSmsCaptcha(sms);
//        if (smsResp == null) {
//            return Response.createError(ResponseCode.SMS_CAPTCHA_INVALID.getMsg(),
//                    ResponseCode.SMS_CAPTCHA_INVALID.getCode());
//        }
//        if (!smsResp.isSuccess()) {
//            return Response.createError(smsResp.getMsg(), smsResp.getCode());
//        }
        //验证卡的状态
        MgRnrCardInfoDTO card = new MgRnrCardInfoDTO();
        card.setIccid(bean.getIccid());
        card.setTenantNo(UserSubjectUtil.getTenantNo());
        Response<MgRnrCardInfoDTO> cardResp = rnrCardInfoClient.getOneByIccid(card);
        if (cardResp == null || !cardResp.isSuccess() || cardResp.getData() == null) {
            return Response.createError(ResponseCode.ICCID_INVALID.getMsg(), ResponseCode.ICCID_INVALID.getCode());
        }
        //判断输入的rnrid 和 iccid查询出来的 rnrId是否一致
        if (!cardResp.getData().getRnrId().equals(bean.getRnrId())) {
            return Response.createError(ResponseCode.INVALID_DATA.getMsg(), ResponseCode.INVALID_DATA.getCode());
        }
        //解绑
        VehicleUnbindDTO unbind = new VehicleUnbindDTO();
        unbind.setTenantNo(UserSubjectUtil.getTenantNo());
        unbind.setIsTrust(0);
        unbind.setIsSecondHandCar(0);
        info.setSerial_number(CuscStringUtils.generateUuid());
        info.setUuid(CuscStringUtils.generateUuid());
        info.setRnrBizzType(RnrBizzTypeEnum.Unbound.getCode());

        unbind.setInfo(info);
        unbind.setOrder(createOrderInfo(info));
        unbind.setCardList(createCardInfo(info, unbind.getOrder().getUuid(), cardResp.getData().getIotId(), cardResp.getData().getIccid()));
        unbind.setCardIds(Arrays.asList(cardResp.getData().getUuid()));
        unbind.setRnrIds(Arrays.asList(cardResp.getData().getRnrId()));
        return unboundClient.batchUnbound(unbind);
    }

    //获取实名工单信息
    private RnrOrderDTO createOrderInfo(MgRnrInfoDTO info) {
        RnrOrderDTO rnrOrderDTO = new RnrOrderDTO();
        rnrOrderDTO.setRnrBizzType(RnrBizzTypeEnum.Unbound.getCode());
        rnrOrderDTO.setOrderType(RnrOrderType.UNBIND.getCode());
        rnrOrderDTO.setRnrId(info.getUuid());
        rnrOrderDTO.setUuid(CuscStringUtils.generateUuid());
        rnrOrderDTO.setTenantNo(info.getTenantNo());
        rnrOrderDTO.setOperator(UserSubjectUtil.getUserId());
        rnrOrderDTO.setCreator(UserSubjectUtil.getUserId());
        rnrOrderDTO.setOrderSource(RnrOrderSourceEnum.H5_CUSTOMER.getCode());
        rnrOrderDTO.setSerialNumber(info.getSerial_number());
        rnrOrderDTO.setOrderStatus(RnrOrderStatusEnum.PASS.getCode());
        rnrOrderDTO.setAutoRnr(true);
        rnrOrderDTO.setAuditType(RnrOrderAuditTypeEnum.AUTO.getCode());
        rnrOrderDTO.setSendWorkOrder(false);
        rnrOrderDTO.setIsBatchOrder(IsBatchEnum.NO_BATCH.getCode());
        return rnrOrderDTO;
    }

    //获取车卡信息
    private List<MgRnrCardInfoDTO> createCardInfo(MgRnrInfoDTO info, String orderId, String iotId, String iccid) {
        List<MgRnrCardInfoDTO> retList = new ArrayList<>();
        MgRnrCardInfoDTO card = new MgRnrCardInfoDTO();
        card.setRnrBizzType(RnrBizzTypeEnum.Unbound.getCode());
        card.setIccid(iccid);
        card.setIotId(iotId);
        card.setRnrId(info.getUuid());
        card.setOrderId(orderId);
        card.setCreator(UserSubjectUtil.getUserId());
        card.setSerial_number(info.getSerial_number());
        card.setUuid(CuscStringUtils.generateUuid());
        card.setTenantNo(info.getTenantNo());
        retList.add(card);
        return retList;
    }

}
