package com.cusc.nirvana.user.rnr.customer.service.impl;

import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.exception.CuscUserException;
import com.cusc.nirvana.user.rnr.customer.service.INewVinCardService;
import com.cusc.nirvana.user.rnr.fp.client.NewVinCardClient;
import com.cusc.nirvana.user.rnr.fp.client.SimVehicleRelClient;
import com.cusc.nirvana.user.rnr.fp.common.ResponseCode;
import com.cusc.nirvana.user.rnr.fp.dto.*;
import com.cusc.nirvana.user.rnr.mg.client.MgRnrCardInfoClient;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrCardInfoDTO;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * @author stayAnd
 * @date 2022/5/18
 */
@Service
public class NewVinCardServiceImpl implements INewVinCardService {

	@Resource
	private NewVinCardClient newVinCardClient;

	@Resource
	private SimVehicleRelClient simVehicleRelClient;

	@Resource
	private MgRnrCardInfoClient cardInfoClient;
	@Override
	public Response<VinCardResultDTO> queryUnBindCardByVin(VinCardQueryDTO queryDTO) {
		MgRnrCardInfoDTO mgRnrCardInfoDTO = new MgRnrCardInfoDTO();
		List<String> list = new ArrayList<>();
		list.add(queryDTO.getVin());
		mgRnrCardInfoDTO.setVinList(list);
		Response<List<MgRnrCardInfoDTO>> mgRnrCardInfo  = cardInfoClient.queryBindListByVin(mgRnrCardInfoDTO);
		if(!mgRnrCardInfo.isSuccess()){
			throw new CuscUserException(mgRnrCardInfo.getCode(),mgRnrCardInfo.getMsg());
		}
		for(MgRnrCardInfoDTO mgRnrCardInfos: mgRnrCardInfo.getData()){
			if(mgRnrCardInfos.getRnrStatus()==0){
				return Response.createError("该车实名审核中");
			}else {
				return Response.createError("该车已实名");
			}
		}
		return newVinCardClient.queryUnBindCardByVin(queryDTO);
	}


	@Override
	public VinCardCheckResultDTO checkVinCard(VinCardCheckDTO vinCardCheckDTO) {

		//paramCheck(vinCardCheckDTO);
		if (!CollectionUtils.isEmpty(vinCardCheckDTO.getIccidList()) && vinCardCheckDTO.getIccidList().stream().distinct().count() != vinCardCheckDTO.getIccidList().size()) {
			throw new CuscUserException(ResponseCode.SYSTEM_ERROR.getCode(),"请不要输入同样的iccid");
		}

		//todo 是否允许补录
		//查询是否允许补录
		IccidListDTO mgRnrCardInfoDTO = new IccidListDTO();

		mgRnrCardInfoDTO.setIccidList(vinCardCheckDTO.getIccidList());

		Response<List<IccidDetailDTO>> iccids = simVehicleRelClient.queryByIccids(mgRnrCardInfoDTO);

		if (!iccids.isSuccess()) {
			throw new CuscUserException(iccids.getCode(), iccids.getMsg());
		}

		for(IccidDetailDTO iccidDetailDTO:iccids.getData()){
			if (!iccidDetailDTO.getRealBySelf()) {
				throw new CuscUserException("", desensitization(iccidDetailDTO.getIccid())+"不可以补录");
			}
		}


		Response<VinCardCheckResultDTO> checkResponse = newVinCardClient.checkVinCard(vinCardCheckDTO);
		if (!checkResponse.isSuccess()) {
			throw new CuscUserException(checkResponse.getCode(),checkResponse.getMsg());
		}


		return checkResponse.getData();
	}

	@Override
	public VinCheckResultDTO checkVin(VinCheckRequestDTO requestDTO) {
		if (requestDTO.getVinList().stream().distinct().count() != requestDTO.getVinList().size()) {
			throw new CuscUserException(ResponseCode.SYSTEM_ERROR.getCode(),"请不要输入同样的iccid");
		}
		Response<VinCheckResultDTO> response = newVinCardClient.checkVin(requestDTO);
		if (!response.isSuccess()) {
			throw new CuscUserException(response.getCode(),response.getMsg());
		}

		//是否允许补录
		if(null != response.getData()) {
			for (VinCheckResultDTO.VinCheckDetailDTO vinCheckDetailDTO : response.getData().getCheckList()) {
				Response<List<VinIccidDTO>> iccids = simVehicleRelClient.getIccidByVin(vinCheckDetailDTO.getVin());

				if (!iccids.isSuccess()) {
					throw new CuscUserException(iccids.getCode(), iccids.getMsg());
				}
				String msg = "";
				for (VinIccidDTO vinIccidDTO : iccids.getData()) {
					if (!vinIccidDTO.getRealBySelf()) {
						msg = msg.concat("当前VIN下iccid为" + desensitization(vinIccidDTO.getIccid()) + "不可以补录;");
					}
				}
				if (!StringUtils.isEmpty(msg)) {
					vinCheckDetailDTO.setErrorMsg(msg);
					vinCheckDetailDTO.setCheckResult(false);
				}
			}
		}
		return response.getData();
	}

	private String desensitization(String parameter){
		String reposen =parameter.substring(0,4)+"***"+parameter.substring(parameter.length()-4,parameter.length());
		return  reposen;

	}

}
