package com.cusc.nirvana.user.rnr.customer.util;

import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.rnr.customer.constants.IdentifyConstants;
import com.cusc.nirvana.user.rnr.fp.common.ResponseCode;
import com.cusc.nirvana.user.rnr.mg.constants.CertTypeEnum;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.Optional;

/**
 * 校验工具类
 *
 * @author yubo
 * @since 2022-04-19 13:49
 */
public class ValidationUtil {

    @Value("${effectiveTime}")
    private String effectiveTime;
    /**
     * 检查手机格式
     *
     * @param phone
     * @return
     */
    public static Response checkPhone(String phone) {
        if (!phone.matches(IdentifyConstants.PHONE_REGEX)) {
            return Response.createError("手机格式错误", ResponseCode.INVALID_DATA.getCode());
        }
        return Response.createSuccess();
    }

    /**
     * 使用正则表达式检查证件合法性
     *
     * @param certType
     * @param identityNumber
     */
    public static Response checkIdentifyNumber(String certType, String identityNumber) {
        Optional<CertTypeEnum> first = Arrays.stream(CertTypeEnum.values())
                .filter(c -> StringUtils.equalsIgnoreCase(c.getCode(), certType))
                .findFirst();
        if (!first.isPresent()) {
            return Response.createError("证件类型错误", ResponseCode.INVALID_DATA.getCode());
        }

        CertTypeEnum certTypeEnum = first.get();
        //如果是身份证,判断号码是否合法
        if (certTypeEnum == CertTypeEnum.IDCARD && !identityNumber.matches(IdentifyConstants.ID_CARD_REGEX)) {
            return Response.createError("身份证号码格式错误", ResponseCode.INVALID_DATA.getCode());
        } else if (certTypeEnum == CertTypeEnum.HKIDCARD && !identityNumber.matches(IdentifyConstants.HK_MACAU_REGEX)) {
            return Response.createError("港澳通行证号码格式错误", ResponseCode.INVALID_DATA.getCode());
        } else if (certTypeEnum == CertTypeEnum.TAIBAOZHENG && !identityNumber.matches(IdentifyConstants.HK_TAIWAN_CITIZEN_REGEX)) {
            return Response.createError("台湾居民来往大陆通行证号码格式错误", ResponseCode.INVALID_DATA.getCode());
        }

        return Response.createSuccess();
    }

    //验证证件照片数量
    //身份证、港澳通行证、台湾通行证 需要正反面两张照片
    public static Response checkIdentifyPics(String certType, int size) {

        if (size < 2) {
            return Response.createError("证件正反面照片不能为空", ResponseCode.INVALID_DATA.getCode());
        }

        return Response.createSuccess();
    }

    /**
     * 判断时间是否在有效期内
     */
    public Response checkEffectiveTime(){

        String startTime = effectiveTime.split("-")[0];
        String endTime = effectiveTime.split("-")[1];

        Calendar cal = Calendar.getInstance();
        int hour = cal.get(Calendar.HOUR_OF_DAY);
        int minute = cal.get(Calendar.MINUTE);
        int second = cal.get(Calendar.SECOND);
        String data = String.valueOf(hour) + ":"+String.valueOf(minute)+ ":"+String.valueOf(second);
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("HH:mm:ss");
        try {
            Date date = simpleDateFormat.parse(data);
            long ts = date.getTime();
            long start =  simpleDateFormat.parse(startTime).getTime();
            long end =  simpleDateFormat.parse(endTime).getTime();

            if(start >= ts || end <= ts){
                return Response.createError(String.valueOf(com.cusc.nirvana.user.rnr.customer.constants.ResponseCode.NO_EFFECTIVE_TIME.getCode()),com.cusc.nirvana.user.rnr.customer.constants.ResponseCode.NO_EFFECTIVE_TIME.getMsg());
            }
            String  res = String.valueOf(ts);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return Response.createSuccess();
    }
}
